#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "I_Config.h"



typedef struct ConfigEntryTag
{
  struct ConfigEntryTag *Next;
  char *Section;
  char *Key;
  char *Value;
  int LineNbr;
} CONFIGENTRY;



I_CONFIG::I_CONFIG ()
{
  initVars ();
}



I_CONFIG::I_CONFIG (const char *CfgName)
{
  initVars ();
  ReadConfig (CfgName);
}



I_CONFIG::~I_CONFIG ()
{
}



void I_CONFIG::initVars ()
{
  CfgList = 0;
  CfgLast = 0;
  TmpEntry = 0;

  TmpSection = 0;
  CurSection = my_strdup ("");
}



char *I_CONFIG::my_strdup (char *s)
{
  char *tmp;

  if((s != NULL) && ((tmp = new char[strlen (s) + 1]) != NULL))
    strcpy(tmp, s);
  else
  {
    fprintf(stderr, "out of mem in my_strdup\n");
    exit(-1);
  }
  return tmp;
}



void I_CONFIG::AddConfigEntry (char *Section, char *Key, char *Value, int Line)
{
  CONFIGENTRY *pCfg;

  if((pCfg = new CONFIGENTRY) == NULL)
  {
    fprintf(stderr, "out of mem in AddConfigEntry\n");
    exit(-1);
  }
  pCfg->Section = Section;
  pCfg->Key     = Key;
  pCfg->Value   = Value;
  pCfg->LineNbr = Line;
  pCfg->Next    = NULL;
  if(CfgList == NULL)
    CfgList = pCfg;
  else
    CfgLast->Next = pCfg;
  CfgLast = pCfg;
}



char *I_CONFIG::StripSpace (char *s)
{
  int	lp;

  for(; (*s == ' ') || (*s == '\n'); ++s);
  if(!*s) return s;
  for(lp = strlen (s) - 1; (lp > 0) &&
                           ((s[lp] == ' ') || (s[lp] == '\n')); lp--)
    s[lp] = '\0';

  return s;
}



char *I_CONFIG::StripQuotes (char *s)
{
  char *q;

  if(strlen (s) < 2)
    return s;

  q = s + strlen (s) - 1;
  if(*s == '\'' && *q == '\'')
  {
    ++s;
    *q = '\0';
  }

  return s;
}



void I_CONFIG::ReadConfig (const char *CfgName)
{
  FILE *f;
  int  Line = 0;
  char buf[132];
  char *s, *p;

  if((f = fopen(CfgName, "r")) == NULL) return;
  while((s = fgets(buf, sizeof(buf), f)) != NULL)
  {
    ++Line;

    s = StripSpace(s);
    if(*s == ';' || *s == '\0')
      continue;                      /* Ignore comments & blank lines */

    if(*s == '[' && s[strlen(s)-1] == ']')
    {
      ++s;
      s[strlen (s) - 1] = '\0';
      CurSection = my_strdup(s);
      continue;
    }

    if((p = strchr(s, '=')) != NULL)
    {                                /* found assignment ? */
      *p++ = '\0';                   /* insert separator */
      p = StripQuotes(StripSpace(p));
      s = StripSpace(s);
    }
    else
    {
      s = StripQuotes(StripSpace(s));
      p = "";
    }

    AddConfigEntry(CurSection, my_strdup(s), my_strdup(p), Line);
  }
  fclose(f);
}



char *I_CONFIG::GetConfigString (char *Section, char *Key, char *Default)
{
  CONFIGENTRY *pCfg;

  for(pCfg = CfgList; pCfg; pCfg = pCfg->Next)
    if(strcmp (Section, pCfg->Section) == 0 && strcmp (Key, pCfg->Key) == 0)
      return pCfg->Value;

  return Default;
}



int I_CONFIG::GetConfigInt (char *Section, char *Key, int Default)
{
  char *s;

  s = GetConfigString(Section, Key, "x");

  return isdigit(*s) ? atoi(s) : Default;
}



bool I_CONFIG::GetConfigBool (char *Section, char *Key, bool Default)
{
  char *s;

  s = GetConfigString(Section, Key, NULL);

  if (s == NULL)
    return Default;

  if (stricmp (s, "NO") == 0)
    return false;

  if (stricmp (s, "YES") == 0)
    return true;
  else
    return Default;
}



char *I_CONFIG::GetFirstEntry (char *Section, char **pKey)
{
  CONFIGENTRY *pCfg;

  for(pCfg = CfgList; pCfg; pCfg = pCfg->Next)
    if (strcmp (Section, pCfg->Section) == 0)
    {
      TmpEntry   = pCfg;
      TmpSection = Section;
      if (pKey)
        *pKey = pCfg->Key;

      return pCfg->Value;
    }

  TmpEntry   = NULL;
  TmpSection = NULL;

  return NULL;
}



char *I_CONFIG::GetNextEntry (char **pKey)
{
  if(TmpEntry == NULL || TmpSection == NULL ||
      TmpEntry->Next == NULL   ||
      strcmp (TmpEntry->Next->Section, TmpSection) != 0)
  {
    TmpEntry   = NULL;
    TmpSection = NULL;

    return NULL;
  }
  TmpEntry = TmpEntry->Next;
  if (pKey)
    *pKey = TmpEntry->Key;

  return TmpEntry->Value;
}



int I_CONFIG::GetLineNbr (void)
{
  if (TmpEntry == NULL)
    return 0;

  return TmpEntry->LineNbr;
}



void I_CONFIG::DiscardConfig ()
{
  char        *HSection;
  CONFIGENTRY *Ptr, *Ptr2;

  if (CfgList == NULL)
    return;

  Ptr = CfgList;
  HSection = Ptr->Section;
  delete HSection;

  do
  {
    if ( Ptr->Section != HSection )
    {
      HSection = Ptr->Section;
      delete HSection;
    }
    delete Ptr->Key;
    delete Ptr->Value;
    Ptr2 = Ptr->Next;
    delete Ptr;
    Ptr = Ptr2;
  } while( Ptr != NULL );

  CfgList  = NULL;
  CfgLast  = NULL;
  TmpEntry = NULL;
}

