#include "TAreaInfo.h"




TAreaInfo::TAreaInfo (const TArea & area, long count, unsigned long size)
: fAreaInfo (new AreaInfoValue (area, count, size))
{
}



TAreaInfo::TAreaInfo (const TAreaInfo & clone)
: fAreaInfo (0)
{
  if (clone.fAreaInfo)
  {
    fAreaInfo = clone.fAreaInfo;
    ++fAreaInfo->fRefCount;
  }
}



TAreaInfo::~TAreaInfo ()
{
  if ((fAreaInfo) && (--fAreaInfo->fRefCount == 0))
    delete fAreaInfo;
}



TAreaInfo & TAreaInfo::operator = (const TAreaInfo & right)
{
  if ((&right != this) && (right.fAreaInfo != fAreaInfo))
  {
    if ((fAreaInfo) && (--fAreaInfo->fRefCount == 0))
      delete fAreaInfo;

    fAreaInfo = right.fAreaInfo;
    ++fAreaInfo->fRefCount;
  }

  return *this;
}



void TAreaInfo::write (ostream & f, TFormattedOutput & format, int inKBytes)
{
  if ((!format.empty ()) && fAreaInfo)
  {
    unsigned long hsize;

    format.setParam (TFormattedOutput::ENTRY_NAME, fAreaInfo->fArea);
    format.setParam (TFormattedOutput::ENTRY_DESC, fAreaInfo->fDesc);
    format.setParam (TFormattedOutput::ENTRY_COUNT, fAreaInfo->fCount);
    hsize = inKBytes == 0 ? fAreaInfo->fSize : (fAreaInfo->fSize +1023) / 1024;
    format.setParam (TFormattedOutput::ENTRY_SIZE, hsize, inKBytes);
    format.writeLine (f);
  }
}



TAreaInfo::AreaInfoValue::AreaInfoValue (const TArea & area, long count, unsigned long size)
: fRefCount (1), fArea (area.fName), fDesc (area.fInfo), fCount (count), fSize (size)
{
}



TAreaInfoList::TAreaInfoList (TFormattedOutput & format)
: fFormat (format), fCount (0), fSize (0), fMinor (0)
{
}



void TAreaInfoList::add (const TAreaInfo & areainfo)
{
  fCount += areainfo.count ();
  fSize += areainfo.size () >> 10;
  fMinor += areainfo.size () & 1023;
  fSize += fMinor >> 10;
  fMinor &= 1023;

  push_back (areainfo);
}



void TAreaInfoList::write (ostream & f, int inKBytes)
{
  if (!fFormat.empty ())
  {
    unsigned long hsize;

    fFormat.setParam (TFormattedOutput::SECTION_COUNT, fCount);
    hsize = inKBytes == 0 ? (fSize << 10) + fMinor : fSize;
    fFormat.setParam (TFormattedOutput::SECTION_SIZE, hsize, inKBytes);
    fFormat.writeTop (f);

    iterator iterArea = begin ();
    while (iterArea != end ())
    {
      (*iterArea).write (f, fFormat, inKBytes);

      ++iterArea;
    }

    fFormat.writeTail (f);
  }
}

