#include "TAreaSelection.h"



void TAreaSelectionList::addEntry (const string & key, const TAreaSort & sort)
{
  bool exclude = key[0] == '-';
  string name = exclude ? key.substr (1) : key;

  push_back (value_type (name, exclude, sort));
}



bool TAreaSelectionList::matchArea (const string & area, TAreaSort & sort) const
{
  const_iterator iter = begin ();
  bool result;

  result = false;
  while (iter != end ())
  {
    if ((*iter).match (area))
    {
      sort = (*iter).sortInfo ();

      result = !((*iter).excluded ());
    }

    ++iter;
  }

  return result;
}



TAreaSelection::TAreaSelection (const string & pattern, bool excluded, const TAreaSort & sort)
: fPattern (pattern), fSort (sort), fExcluded (excluded)
{
  for (int i = 0; i < fPattern.length (); i++)
    fPattern[i] = toupper (fPattern[i]);
}



/************************************************************************/
/* Matching routine :                                                   */
/*                                                                      */
/* Compares a string against a pattern.                                 */
/* Following patterns are valid :                                       */
/*                                                                      */
/*   ABCDEF                     // no wildcard                          */
/*   AB??EF                     // placeholder for 1 character          */
/*   AB*                        // wildcard for any ending              */
/*   AB??E*                     // mixed                                */
/*                                                                      */
/* Returns False on failure and True on success.                        */
/************************************************************************/

bool TAreaSelection::match (const string & str) const
{
  int loop;
  int compare;
  int len;

  len = str.length ();
  for (loop = 0, compare = 0; loop < fPattern.length (); loop++)
    switch (fPattern[loop])
    {
      case '*':
        return true;

      case '?':
        if (compare >= len)
          return false;
       compare++;
       break;

      default:
       if (compare >= len)
         return false;

       if (fPattern[loop] != str[compare])
         return false;

       compare++;
       break;
    }

  if (fPattern.length () < len)
    return false;

  return true;
}

