#include "TConfig.h"

#include "Macros.h"
#include <stdio.h>
#include <stdlib.h>
#include "I_Config.h"
#include "TMaxSeverity.h"




#define SECONDSPERDAY 86400L


typedef char *PStr;

enum Sections
{
  PATHS_SECTION,
  STYLE_SECTION,
  GLOBAL_SECTION,
  AREAS_SECTION,
  TEXTFILEAREA_SECTION,
  SUMMARY_SECTION,
  AREADEFINITION_SECTION,
  FILESBBS_SECTION,
  HTMLFILEAREA_SECTION,
  HTMLAREA_SECTION,
  HTMLDIRMAPPING_SECTION,
  LAST_SECTION              // to indicate the maximum index
} Sections;

char *Sect[LAST_SECTION] = {"Paths",
                            "Style",
                            "Global",
                            "Areas",
                            "TextFileArea",
                            "Summary",
                            "AreaDefinition",
                            "FILESBBS",
                            "HtmlFileArea",
                            "HtmlArea",
                            "HtmlDirMapping"};

PStr Keys[] =    {"MaxDir",
                  "AllFiles",
                  "AllHeader",
                  "NewFiles",
                  "NewHeader",
                  "WordBreak",
                  "--unused--",
                  "PurgeSize",
                  "--unused--",
                  "--unused--",
                  "InKbytes",
                  "MarkNew",
                  "Privilege",
                  "Summary",
                  "Sort",
                  "RecentNew",
                  "FileSort",
                  "MultilineTag",
                  "MultilineIndent",
                  "--unused--",
                  "SkipLines",
                  "BbsName",
                  "HtmlFilesDir",
                  "GenerateHtml"};

PStr SortKey[] = {"None",
                  "File",
                  "Date"};

PStr SortTyp[] = {"Ascending",
                  "Descending"};

PStr defTextFileArea[] =
  {"ͻ",
   " Area : %IIII%  %ddddddddddddddddddddddddddddddddddddddddddddddddddddddd% ",
   "ͼ",
   "%",
   "%"};
PStr defHtmlAreaHeader[] =
  {"%", "%", "%"};
PStr defHtmlAreaTrailer[] = { "%", "%", "%"};
PStr defSummHeader[] =
  {"                                 ",
   "                             ",
   "                                     ",
   "%",
   "Ŀ",
   " List of fileareas :                                                         ",
   "                                                                             ",
   " Area   Description                                     Files  used Space ",
   "Ĵ",
   "%IIII%  %dddddddddddddddddddddddddddddddddddddddddddd% %NNNN% %KKKKKKKK%K ",
   "Ĵ",
   " Summary                                                %XXXX% %SSSSSSSS%K ",
   ""};





TConfig::TConfig (const string & path)
: fDefaultSorting (TAreaSort::NoSorting, TAreaSort::SortAscending)
{
  loadConfig (path);
}



TConfig::TConfig ()
{
}



TConfig::~TConfig ()
{
}



void TConfig::loadConfig (const string & path)
{
  char *s;
  char *p;

  if (path.length () == 0)
    conf = new I_CONFIG ("FFL.CFG");
  else
    conf = new I_CONFIG (path.c_str ());

  fBbsName = conf->GetConfigString (Sect[GLOBAL_SECTION], Keys[21], "BBS");
  fMaximusDir = conf->GetConfigString (Sect[PATHS_SECTION], Keys[0], ".");
  if (fMaximusDir.length () == 0)
    fMaximusDir = ".";
  if (fMaximusDir[fMaximusDir.length () - 1] != PATH_DELIMITER_CHAR)
    fMaximusDir += PATH_DELIMITER;
  TMaxSeverity::init (fMaximusDir);

  fAlllistPath = conf->GetConfigString (Sect[PATHS_SECTION], Keys[1], "22474488.TXT");
  fAllinfoPath = conf->GetConfigString (Sect[PATHS_SECTION], Keys[2], "");
  fNewlistPath = conf->GetConfigString (Sect[PATHS_SECTION], Keys[3], "");
  fNewinfoPath = conf->GetConfigString (Sect[PATHS_SECTION], Keys[4], "");
  fHtmlFilesDir = conf->GetConfigString (Sect[PATHS_SECTION], Keys[22], ".");
  if (fHtmlFilesDir.length () == 0)
    fHtmlFilesDir = ".";
  if (fHtmlFilesDir[fHtmlFilesDir.length () - 1] != PATH_DELIMITER_CHAR)
    fHtmlFilesDir += PATH_DELIMITER;
  fGenerateHtml = conf->GetConfigBool (Sect[GLOBAL_SECTION], Keys[23], True);

  fWordbreak = conf->GetConfigInt (Sect[STYLE_SECTION], Keys[5], 0x7FFF);
  if (fWordbreak < 20)
    fWordbreak = 20;
  fPrivilege = TMaxSeverity::ascii2severity (conf->GetConfigString (Sect[GLOBAL_SECTION], Keys[12], "Normal"));
  fNoSummary = !conf->GetConfigBool (Sect[GLOBAL_SECTION], Keys[13], true);
  fSkipLines = conf->GetConfigInt (Sect[STYLE_SECTION], Keys[20], 0);
  fPurgeSize = conf->GetConfigBool (Sect[STYLE_SECTION], Keys[7], false);
  fNewfileMarker = conf->GetConfigString (Sect[STYLE_SECTION], Keys[11], "*");
  fInKBytes = conf->GetConfigBool (Sect[STYLE_SECTION], Keys[10], true);
  fMustSort = conf->GetConfigBool (Sect[GLOBAL_SECTION], Keys[14], false);
  int days = conf->GetConfigInt (Sect[GLOBAL_SECTION], Keys[15], 7);
  fRecentNew = SECONDSPERDAY * (long) days;

  cMultiTag = *(conf->GetConfigString (Sect[FILESBBS_SECTION], Keys[17], " "));
  multiIndent = conf->GetConfigInt (Sect[FILESBBS_SECTION], Keys[18], 31);

  s = conf->GetConfigString (Sect[GLOBAL_SECTION], Keys[18], "");
  fDefaultSorting = getSortInfo (s, fDefaultSorting);

  if ((p = conf->GetFirstEntry (Sect[AREAS_SECTION], &s)) != NULL)
  {
    TAreaSort defSort = fDefaultSorting;

    while ((p != NULL) && (strlen (s) > 0))
    {
      fAreaSelectionList.addEntry (s, getSortInfo (p, fDefaultSorting));
      p = conf->GetNextEntry (&s);
    }
  }

  readHeaderDefs (fTextFileArea, Sect[TEXTFILEAREA_SECTION], defTextFileArea, 3, 1, 1);
  readHeaderDefs (fSummaryDef, Sect[SUMMARY_SECTION], defSummHeader, 9, 1, 3);
  readHeaderDefs (fHtmlFileArea, Sect[HTMLFILEAREA_SECTION], defHtmlAreaHeader, 1, 1, 1);
  readHeaderDefs (fHtmlArea, Sect[HTMLAREA_SECTION], defHtmlAreaTrailer, 1, 1, 1);

  if ((p = conf->GetFirstEntry (Sect[AREADEFINITION_SECTION], &s)) != NULL)
  {
    while ((p != NULL) && (strlen (s) > 0))
    {
      fAdditionalAreas.push_back (TAdditionalArea (s, p));
      p = conf->GetNextEntry (&s);
    }
  }

  if ((p = conf->GetFirstEntry (Sect[HTMLDIRMAPPING_SECTION], &s)) != NULL)
  {
    while ((p != NULL) && (strlen (s) > 0))
    {
      fHtmlDirMapper.addMapping (s, p);

      p = conf->GetNextEntry (&s);
    }
  }

  delete conf;
}



void TConfig::readHeaderDefs (TFormattedOutput & list, char *section, char **def,
                              int headerLines, int bodyLines, int trailerLines)
{
  char      *key;
  char      *line;
  int       loop;
  int       count;

  if ((line = conf->GetFirstEntry (section, &key)) != NULL)
    while ((line != NULL) && (strlen (key) > 0))
    {
      list.insert (*key, line);
      line = conf->GetNextEntry (&key);
    }
  else                                /* if nothing in input, set defaults */
  {
    for (loop = 0; loop < headerLines; loop++)
      list.insert ('H', def[loop]);
    for (count = 0; count < bodyLines; count++, loop++)
      list.insert ('L', def[loop]);
    for (count = 0; count < trailerLines; count++, loop++)
      list.insert ('T', def[loop]);
  }
}



TAreaSort TConfig::getSortInfo (const char *str, const TAreaSort & defSort)
{
  char *buf;
  char *ptr;

  TAreaSort::SortKey       key   = defSort.getSortKey ();
  TAreaSort::SortDirection order = defSort.getSortOrder ();

  if (strlen (str) == 0)
    return defSort;

  buf = new char (strlen (str));
  strcpy (buf, str);

  if ((ptr = strchr (buf, ',')) != NULL)
  {                                       // ordering specified
    *ptr = '\0';
    for (ptr++; (*ptr != '\0') && (*ptr == ' '); ptr++);
    if (strnicmp (ptr, SortTyp[1], strlen (SortTyp[1])) == 0)
      order = TAreaSort::SortDescending;
    else
      if (strnicmp (ptr, SortTyp[0], strlen (SortTyp[0])) == 0)
        order = TAreaSort::SortAscending;
  }

  for (ptr = buf; (*ptr != '\0') && (*ptr == ' '); ptr++);
  if (strnicmp (ptr, SortKey[1], strlen (SortKey[1])) == 0)
    key = TAreaSort::SortByFile;
  else
    if (strnicmp (ptr, SortKey[2], strlen (SortKey[2])) == 0)
      key = TAreaSort::SortByDate;
    else
      if (strnicmp (ptr, SortKey[0], strlen (SortKey[0])) == 0)
        key = TAreaSort::NoSorting;

  delete [] buf;

  return TAreaSort (key, order);
}



int TConfig::treatCmdLine (char *par)
{
  if ((par == NULL) || (strlen (par) < 3) || (*par != '/'))
    return false;

  strupr (par);

  switch (par[1])
  {
    case 'B' : fWordbreak = atoi (&par[2]);
               if (fWordbreak < 60)
                 return false;
               break;

    case 'C' : break;              /* config file already treated */

    case 'D' : fRecentNew = SECONDSPERDAY * atoi (&par[2]);
               break;

    case 'I' : if (strlen (par) < 4)
                 return false;

               switch (par[2])
               {
                 case 'A' : fAllinfoPath = &par[3];
                            break;

                 case 'N' : fNewinfoPath = &par[3];
                            break;

                 default  : return false;
               }
               break;

    case 'K' : if (par[2] == 'B')
                 fInKBytes = 1;
               else
                 return false;
               break;

    case 'L' : if (strlen (par) < 4)
                 return false;

               switch (par[2])
               {
                 case 'A' : fAlllistPath = &par[3];
                            break;

                 case 'N' : fNewlistPath = &par[3];
                            break;

                 default  : return false;
               }
               break;

    case 'N' : switch (par[2])
               {
                 case 'A' : fMustSort = 0;
                            break;

                 case 'S' : fNoSummary = 1;
                            break;

                 default  : return false;
               }
               break;

    case 'M' : fMaximusDir = &par[2];
               if (fMaximusDir[fMaximusDir.length () - 1] != PATH_DELIMITER_CHAR)
                 fMaximusDir += PATH_DELIMITER;
               break;

    case 'P' : if (par[2] == 'S')
                 fPurgeSize = 1;
               else
                 return false;
               break;

    case 'S' : fPrivilege = TMaxSeverity::ascii2severity (&par[2]);
               break;

    default  : return false;
  }

  return true;
}

