#ifndef __T_CONFIG_H__
#define __T_CONFIG_H__


#include <time.h>
#include <string>
#include "TFormattedOutput.h"
#include "TAreaSelection.h"
#include "TAdditionalArea.h"
#include "THtmlDirMapper.h"

// forward references
class I_CONFIG;



class TConfig
{
  public:
    TConfig (const string & path);
    TConfig ();
    ~TConfig ();

    void loadConfig (const string & path);

    bool treatCmdLine (char *parameter);

    const string & getBbsName () const { return fBbsName; }
    bool getGenerateHtml () const { return fGenerateHtml; }
    int getPriv () { return fPrivilege; };
    time_t getRecent () { return fRecentNew; };
    int getSkipLines () const { return fSkipLines; }
    const string & newFileMarker () { return fNewfileMarker; };
    bool areInKBytes () { return fInKBytes; };
    bool mustPurgeSize () { return fPurgeSize; };
    int getWordBreak () { return fWordbreak; };
    bool isSorted () { return fMustSort; };
    bool hasNoSummary () { return fNoSummary; };
    const TAreaSort defaultSorting () { return fDefaultSorting; }
    const string & getMaxDir () { return fMaximusDir; };
    const string & getAllFiles () { return fAlllistPath; };
    const string & getAllHeader () { return fAllinfoPath; };
    const string & getNewFiles () { return fNewlistPath; };
    const string & getNewHeader () { return fNewinfoPath; };
    const string & getHtmlFilesDir () const { return fHtmlFilesDir; }
    char getMultilineTag () { return cMultiTag; }
    int getMultilineIndent () { return multiIndent; }
    const TAreaSelectionList & getAreas () { return fAreaSelectionList; };
    const TAdditionalAreaList & getAddAreas () { return fAdditionalAreas; }

    TFormattedOutput & getTextFileAreaFormat () { return fTextFileArea; };
    TFormattedOutput & getSummaryDef () { return fSummaryDef; };
    TFormattedOutput & getHtmlFileAreaFormat () { return fHtmlFileArea; };
    TFormattedOutput & getHtmlAreaFormat () { return fHtmlArea; };

    const THtmlDirMapper & getHtmlDirMapper () const { return fHtmlDirMapper; }

  private:
    void readHeaderDefs (TFormattedOutput & list, char *section, char **def,
                         int headerLines, int bodyLines, int trailerLines);
    TAreaSort getSortInfo (const char *str, const TAreaSort & defSort);

    I_CONFIG                 *conf;
    int                      count;

    string                   fMaximusDir;
    string                   fAlllistPath;
    string                   fAllinfoPath;
    string                   fNewlistPath;
    string                   fNewinfoPath;
    string                   fHtmlFilesDir;
    string                   fBbsName;
    string                   fNewfileMarker;
    char                     cMultiTag;
    int                      multiIndent;
    int                      fWordbreak;
    int                      fPrivilege;
    int                      fSkipLines;
    bool                     fGenerateHtml;
    bool                     fNoSummary;
    bool                     fPurgeSize;
    bool                     fInKBytes;
    bool                     fMustSort;
    TAreaSort                fDefaultSorting;
    time_t                   fRecentNew;
    TAreaSelectionList       fAreaSelectionList;
    TFormattedOutput         fTextFileArea;
    TFormattedOutput         fSummaryDef;
    TFormattedOutput         fHtmlFileArea;
    TFormattedOutput         fHtmlArea;
    TAdditionalAreaList      fAdditionalAreas;
    THtmlDirMapper           fHtmlDirMapper;
};


#endif
