#include "TFileEntry.h"
#include <sys/stat.h>



TFileEntry::TFileEntry ()
: fSize (0), fDay (0), fMonth (0), fYear (0), fFileTime (0)
{
}



TFileEntry::TFileEntry (const string & path, const string & name,
                        const string & desc, const TAreaSort & sort)
: fName (name), fDesc (desc), fSort (sort), fSize (0), fDay (0), fMonth (0), fYear (0), fFileTime (0)
{
  if (path.length () != 0)
    getFileSizeDate (path);
}



TFileEntry::TFileEntry (const TFileEntry & clone)
: fName (clone.fName), fDesc (clone.fDesc), fSize (clone.fSize),
  fDay (clone.fDay), fMonth (clone.fMonth), fYear (clone.fYear),
  fSort (clone.fSort), fFileTime (clone.fFileTime)
{
}



bool TFileEntry::operator == (const TFileEntry & right) const
{
  return fName == right.fName;
}



bool TFileEntry::operator < (const TFileEntry & right) const
{
  bool result;

  result = true;
  switch (fSort.getSortKey ())
  {
    case TAreaSort::SortByFile:
      result = fName < right.fName;
      break;

    case TAreaSort::SortByDate:
      result = ((fYear < right.fYear) ||
                ((fYear == right.fYear) && ((fMonth < right.fMonth) ||
                 ((fMonth == right.fMonth) && (fDay == right.fDay)))));
      break;
  }

  if (fSort.getSortOrder () == TAreaSort::SortAscending)
    result = !result;

  return result;
}



bool TFileEntry::operator > (const TFileEntry & right) const
{
  bool result;

  result = true;
  switch (fSort.getSortKey ())
  {
    case TAreaSort::SortByFile:
      result = fName > right.fName;
      break;

    case TAreaSort::SortByDate:
      result = ((fYear > right.fYear) ||
                ((fYear == right.fYear) && ((fMonth > right.fMonth) ||
                 ((fMonth == right.fMonth) && (fDay == right.fDay)))));
      break;
  }

  if (fSort.getSortOrder () == TAreaSort::SortAscending)
    result = !result;

  return result;
}



TFileEntry & TFileEntry::operator = (const TFileEntry & right)
{
  if (&right != this)
  {
    fName = right.fName;
    fDesc = right.fDesc;
    fSize = right.fSize;
    fDay = right.fDay;
    fMonth = right.fMonth;
    fYear = right.fYear;
    fSort = right.fSort;
    fFileTime = right.fFileTime;
  }

  return *this;
}



void TFileEntry::purgeSize ()
{
  int loop;

  if ((atol (fDesc.c_str ()) == fSize) && (isdigit (fDesc[0]) != 0))
  {
    for (loop = 0; (loop < fDesc.length ()) && (fDesc[loop] != ' '); loop++);
    for (; (loop < fDesc.length ()) && (fDesc[loop] == ' '); loop++);
    if (loop > 0)
      fDesc.erase (0, loop);
  }
}



void TFileEntry::appendDesc (const string & desc)
{
  fDesc += "\n";
  fDesc += desc;
}



void TFileEntry::getFileSizeDate (const string & path)
{
  struct stat info;
  struct tm *timestruct;
  string fpath = path + fName;

  if (stat (fpath.c_str (), &info) == 0)
  {
    fSize = info.st_size;
    fFileTime = info.st_mtime;
    timestruct = localtime (&(info.st_mtime));
    fDay = timestruct->tm_mday;
    fMonth = timestruct->tm_mon + 1;
    fYear = timestruct->tm_year + 1900;
  }
  else
    fFileTime = 0;
}

