#include "TFileList.h"
#include <ctype.h>
#include <fstream>



TFileList::TFileList (const string & path, const string & fbbs, int skipLines,
                      bool killsize, time_t runtime, time_t agenew,
                      const TAreaSort & sort, char tagChar, int bbsIndent)
: fKillSize (killsize), fSumSize (0), fSumFiles (0), fNewSize (0), fNewFiles (0),
  fRunTime (runtime), fAgeNew (agenew), fSort (sort)
{
  ifstream filesBbsFile (fbbs.c_str (), ios::in);
  if (filesBbsFile)
  {
    string linebuffer;
    TFileEntry fileEntry;
    bool entryAvail = false;

    long count = 0;
    while (filesBbsFile.good ())
    {
      getline (filesBbsFile, linebuffer);        // read one line including preceding blancs
      count++;
      if ((count % 10) == 0)
        cout << "Line " << count << "\r" << flush;

      if (count <= skipLines)
        continue;

      for (int loop = 0; loop < linebuffer.length (); loop++)
        if (linebuffer[loop] == '\t')
          linebuffer[loop] = ' ';

      if ((linebuffer.length () != 0) &&
          ((isalpha (linebuffer[0]) != 0)  || (isdigit (linebuffer[0]) != 0) ||
           (linebuffer[0] == '_') || (linebuffer[0] == '&') ||
           (linebuffer[0] == '$') || (linebuffer[0] == '!') ||
           (linebuffer[0] == '#')))
      {
        if (entryAvail)
        {
          push_back (fileEntry);
          entryAvail = false;
        }

        entryAvail = handleFBbsEntry (fileEntry, path, linebuffer);
      }
      else
      {
        if ((linebuffer.length () < bbsIndent) ||
            ((tagChar != ' ') && (linebuffer[0] != tagChar)) ||
            ((tagChar == ' ') && (linebuffer.find_first_not_of (' ') < bbsIndent)))
        {
          if (entryAvail)
          {
            push_back (fileEntry);
            entryAvail = false;
          }
        }

        if (!entryAvail)
          push_back (TFileEntry ("", "", linebuffer, fSort));
        else
          fileEntry.appendDesc (linebuffer.substr (bbsIndent));
      }
    }
    if (entryAvail)
      push_back (fileEntry);    // append unsaved file entry

    cout << "Line " << count << "\r" << flush;

    // sort list on request
    if (fSort.getSortKey () != TAreaSort::NoSorting)
      TFileList::sort ();
  }
}



bool TFileList::handleFBbsEntry (TFileEntry & entry, const string & path, const string & line)
{
  int pos;
  string name;
  string options;
  string desc;

  if ((pos = line.find_first_of (' ')) != string::npos)
  {
    name = line.substr (0, pos);         // obtain file name

    if ((pos = line.find_first_not_of (' ', pos)) != string::npos)
    {
      if (line[pos] == '/')              // display modifier of Maximus ?
      {
        int optPos = pos;

        if ((pos = line.find_first_of (' ', pos)) != string::npos)
          options = line.substr (optPos, pos - optPos);
        else
          pos = optPos;
      }
      desc = line.substr (pos);
    }
  }

  entry = TFileEntry (path, name, desc, fSort);

  fSumFiles++;
  fSumSize += entry.size ();

  // calc if this file is newer than <fAgeNew>
  if ((fRunTime - entry.fileTime ()) <= fAgeNew)
  {
    fNewFiles++;
    fNewSize += entry.size ();
  }

  // calc if it is newer than the up to now most recent one
  if ((size () == 0) || (fLatestFile.fileTime () < entry.fileTime ()))
    fLatestFile = entry;

  // some areas contain the file size as part of the description
  if (fKillSize)
    entry.purgeSize ();

  return true;
}

