#include "TFormattedOutput.h"
//#include <stdarg.h>
#include <string.h>
#include <string>
#include <iomanip>
#include <stdlib.h>



// the size must exactly match TFormattedOutput::Keys
static char KeyLetters[] = "ACEGIKMOQSUWY024";
static char OptLetters[] = "BDFHJLNPRTVXZ135";



class ParamEntry
{
  public:
    ParamEntry () {}

    virtual int size () const = 0;
//    virtual void printParam (FILE *f, int width, int align) = 0;
    virtual void printParam (ostream & f, int width, bool align, bool extendible) = 0;
};



class ParamEntryString : public ParamEntry
{
  public:
    ParamEntryString (const string & value) : fValue (value) {}
    ParamEntryString (const char *value) : fValue (value) {}

    virtual int size () const { return fValue.length (); }
    virtual void printParam (ostream & f, int width, bool align, bool extendible);

  private:
    string fValue;
};



void ParamEntryString::printParam (ostream & f, int width, int align, bool extendible)
{
  if (extendible)
    if ((width <= fValue.length ()) || align)
      f << fValue;
    else
      f << setw (width) << setiosflags (ios::left) << fValue;
//    f << setw (width > fValue.length () ? width : fValue.length ()) << fValue;
  else
    if (width < fValue.length ())
      f << setw (width) << fValue.substr (0, width);
    else
      if (width == fValue.length ())
        f << fValue;
      else
        if (align)
          f << setw (width - fValue.length ()) << " " << fValue;
        else
          f << fValue << setw (width - fValue.length ()) << " ";
}


/*
class ParamEntryDWord : public ParamEntry
{
  public:
    ParamEntryDWord (unsigned long value, bool inK) : fValue (value), fInK (inK) {}

    virtual int size () const { return 10; }
    virtual void printParam (ostream & f, int width, int align, bool extendible);

  private:
    unsigned long fValue;
    bool fInK;
};



void ParamEntryDWord::printParam (ostream & f, int width, int align, bool extendible)
{
  if (extendible)
  {
    char buffer[33];

    _ultoa (fValue, buffer, 10);
    strcat (buffer, "K");

    f << setw (max (width, atrlen (buffer))) << setiosflags (ios::left) << fValue;
  else
    f << setw (width - (fInK ? 1 : 0)) << setiosflags (align ? ios::right : ios::left) << fValue;

  if (fInK)
    f << "K";
}
*/


TFormattedOutput::TFormattedOutput ()
: fEmpty (true)
{
}



TFormattedOutput::~TFormattedOutput ()
{
  paramList::iterator iter = fParams.begin ();
  while (iter != fParams.end ())
  {
    delete (*iter).second;

    ++iter;
  }
}



void TFormattedOutput::insert (char key, const string & item)
{
  fEmpty = false;

  switch (key)
  {
    case 'H':
    case 'h': addtolist (fHeader, item);
              break;

    case 'L':
    case 'l': addtolist (fList, item);
              break;

    case 'O':
    case 'o': addtolist (fOptional, item);
              break;

    case 'T':
    case 't': addtolist (fTrailer, item);
              break;
  }
}



void TFormattedOutput::setParam (Keys key, const char *value)
{
  paramList::iterator iter;
//  ParamEntry *entry = new ParamEntryString (value);

  if ((iter = fParams.find (KeyLetters[key])) != fParams.end ())
  {                         // key found - kill old entry
    delete (*iter).second;
    fParams.erase (iter);
  }

  if (value && (*value != '\0'))
    fParams.insert (paramList::value_type (KeyLetters[key], new ParamEntryString (value)));
}



void TFormattedOutput::setParam (Keys key, const string & value)
{
  paramList::iterator iter;
//  ParamEntry *entry = new ParamEntryString (value);

  if ((iter = fParams.find (KeyLetters[key])) != fParams.end ())
  {                         // key found - kill old entry
    delete (*iter).second;
    fParams.erase (iter);
  }

  if (value.length () != 0)
    fParams.insert (paramList::value_type (KeyLetters[key], new ParamEntryString (value)));
}



void TFormattedOutput::setParam (Keys key, unsigned long value, bool inK)
{
  char buffer[34];

  _ultoa (value, buffer, 10);              // must be adapted for UNIX !!!!
  if (inK)
    strcat (buffer, "K");

  paramList::iterator iter;
  ParamEntry *entry = new ParamEntryString (buffer);

  if ((iter = fParams.find (KeyLetters[key])) != fParams.end ())
  {                         // key found - kill old entry
    delete (*iter).second;
    fParams.erase (iter);
  }

  fParams.insert (paramList::value_type (KeyLetters[key], entry));
}



void TFormattedOutput::writeTop (ostream & f)
{
  writeSection (f, fHeader);
}



void TFormattedOutput::writeLine (ostream & f, bool optional)
{
  if (optional)
    writeSection (f, fOptional);
  else
    writeSection (f, fList);
}



void TFormattedOutput::writeTail (ostream & f)
{
  writeSection (f, fTrailer);
}



void TFormattedOutput::writeSection (ostream & f, stringVector & list)
{
  stringVector::iterator i;

  for (i = list.begin (); i != list.end (); ++i)
    writeRow (f, *i);
}



void TFormattedOutput::addtolist (stringVector & list, const string & item)
{
  list.push_back (item);
}



void TFormattedOutput::writeRow (ostream & f, const string & line)
{
  char *ptr;
  char *start;
  char *buff;
  int  count;

  buff = new char [line.length () + 1];
  strcpy (buff, line.c_str ());
  if ((line.length () > 1) && (buff[0] == '%') && (buff[1] == ' '))
    buff[0] = ' ';
  start = buff;
  while ((ptr = strchr (start, '%')) != NULL)
  {
    *ptr = '\0';
    f << start;
    ptr++;
    for (start = ptr, count = 2;
         (*start != '\0') && (*start != '%'); start++, count++);
    if (*start == '\0')                    /* up to end of string, no token */
      f << ptr;
    else
    {
      if (start == ptr)                    /* replacement for '%' */
        f << "%";
      else                                 /* normal token */
        printParm (f, toupper (*ptr), count, isupper (*ptr) != 0);
      start++;
    }
  }
  f << start << endl;

  delete [] buff;
}



ParamEntry *TFormattedOutput::getParm (char key)
{
  paramList::iterator iter;

  if ((iter = fParams.find (key)) != fParams.end ())
    return (*iter).second;

  return 0;
}



void TFormattedOutput::printParm (ostream & f, char key, int width, bool align)
{
  ParamEntry *parm;
  bool extendible;

  if (extendible = (strchr (OptLetters, key) != 0))
    key--;

  parm = getParm (key);
  if (parm == 0)
    f << setw (key == '2' ? 1 : width) << " ";
  else
    parm->printParam (f, key == '2' ? 1 : width, align, extendible);
}

