#include "THtmlDirMapper.h"



THtmlDirMapperEntry::THtmlDirMapperEntry (const string & original, const string & replace)
: fOriginal (original), fReplace (replace)
{
}



THtmlDirMapperEntry::THtmlDirMapperEntry (const THtmlDirMapperEntry & clone)
: fOriginal (clone.fOriginal), fReplace (clone.fReplace)
{
}



THtmlDirMapperEntry & THtmlDirMapperEntry::operator = (const THtmlDirMapperEntry & right)
{
  if (&right != this)
  {
    fOriginal = right.fOriginal;
    fReplace = right.fReplace;
  }

  return *this;
}



bool THtmlDirMapperEntry::operator == (const THtmlDirMapperEntry & right) const
{
  return fOriginal == right.fOriginal;
}



bool THtmlDirMapperEntry::operator < (const THtmlDirMapperEntry & right) const
{
  return fOriginal < right.fOriginal;
}


bool THtmlDirMapperEntry::match (const string & path) const
{
  return (fOriginal.length () <= path.length ()) &&
         (path.substr (0, fOriginal.length ()) == fOriginal);
}



void THtmlDirMapper::addMapping (const string & original, const string & replace)
{
#ifndef UNIX
  string compPath = original;

  for (int loop = 0; loop < compPath.length (); loop++)
    compPath[loop] = toupper (compPath[loop]);

  fMapperList.push_back (THtmlDirMapperEntry (compPath, replace));
#else
  fMapperList.push_back (THtmlDirMapperEntry (original, replace));
#endif
}



bool THtmlDirMapper::mapPath (const string & path, string & newPath) const
{
  bool mappingFound = false;

  newPath = path;

#ifndef UNIX
  for (int loop = 0; loop < newPath.length (); loop++)
    newPath[loop] = toupper (newPath[loop]);
#endif
  mapperList::const_iterator iterMap = fMapperList.begin ();
  while (iterMap != fMapperList.end ())
  {
    if ((*iterMap).match (newPath))
    {
      newPath.erase (0, (*iterMap).original ().length ());
      newPath.insert (0, (*iterMap).replacement ());

      mappingFound = true;

      break;
    }

    ++iterMap;
  }

#ifndef UNIX
  // if there is a drive letter replace the colon by a pipe-mark and
  // prefix it with 'file:///
  // thos works with netscape - other browsers must be checked

  int pos;
  if ((pos = newPath.find_first_of (':')) != string::npos)
  {                                          // DOS path style found !
    newPath[pos] = '|';
    newPath.insert (0, "file:///");
  }

  // replace the back-slashes by forward slashes
  pos = 0;
  while ((pos = newPath.find_first_of ('\\', pos + 1)) != string::npos)
    newPath[pos] = '/';
#endif

  return mappingFound;
}

