#include "THtmlFileWriter.h"



THtmlFileWriter::THtmlFileWriter (ostream *file, bool inKBytes,
                                  char fileMarker, TFormattedOutput & format)
: TFileWriter (file), fInKBytes (inKBytes), fFileMarker (fileMarker), fFormat (&format)
{
}



void THtmlFileWriter::write (const TFileEntry & entry, bool isNew)
{
  int columns;

  if (fFile == 0)
    return;

  if (!fFormat || fFormat->empty ())
    return;

  if (entry.name ().length () == 0)
  {
    fFormat->setParam (TFormattedOutput::ENTRY_DESC, entry.desc ().c_str ());
    fFormat->writeLine (*fFile, true);
  }
  else
  {
    char marker[2];
    char date[9];

    *marker = isNew ? fFileMarker : ' ';
    marker[1] = '\0';
    sprintf (date, "%02i.%02i.%02i", entry.day (), entry.month (), entry.year () % 100);

    fFormat->setParam (TFormattedOutput::ENTRY_NAME, entry.name ().c_str ());
    fFormat->setParam (TFormattedOutput::ENTRY_DATE, date);
    fFormat->setParam (TFormattedOutput::ENTRY_SIZE, fInKBytes ? (entry.size () + 1023) / 1024 : entry.size (), fInKBytes);
    fFormat->setParam (TFormattedOutput::ENTRY_NEWMARKER, marker);

    int next;
    int start = 0;
    string comment;
    do
    {
      if (start != 0)
        comment += "<br>";

      next = entry.desc ().find ('\n', start);

      comment += "\n      " + entry.desc ().substr (start, next == string::npos ? next : next - start);
      start = next + 1;
    } while (next != string::npos);
    fFormat->setParam (TFormattedOutput::ENTRY_DESC, comment.c_str ());

    fFormat->writeLine (*fFile);
  }
}


