#include "TMaxSeverity.h"
#include "Macros.h"
#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>



TMaxSeverity::severityValueMap  TMaxSeverity::fValues;
TMaxSeverity::severityStringMap TMaxSeverity::fStrings;


// Structure definition of ACCESS.DAT file of Maximus CBCS
//
// first a header (size 12 Bytes) defines the global layout of the file
//
// typedef struct
// {
//   unsigned long  id;
//   unsigned short first_rec;
//   unsigned short number_recs;
//   unsigned short rec_size;
//   unsigned short str_size;
// } HEADER;
//
//
// Now there a number (defined in HEADER.number_recs) of severity definition records.
// The record size is defined in HEADER.rec_size which starts at offset HEADER.first_rec.
//
// typedef struct
// {
//   unsigned short usLevel;
//   unsigned short usKey;
//
//   unsigned short abbrev_offset;
//   unsigned short desc_offset;
//   unsigned short alias_offset;
//
//   unsigned short usTimeDay;
//   unsigned short usTimeCall;
//   unsigned short usCallsDay;
//   unsigned short usMinBaud;
//   unsigned short usFileBaud;
//   unsigned short usFileRatio;
//   unsigned short usFreeRatio;
//   unsigned long  ulFileLimit;
//   unsigned short usUploadReward;
//
//   unsigned short loginfile_offset;
//
//   unsigned long  ulAccFlags;
//   unsigned long  ulMailFlags;
//   unsigned long  ulUsrFlags;
//
//   unsigned short usOldPriv;
// } RECORD;
//
// After all records there follows a list of zero terminated strings.
// The size of this block is defined in HEADER.str_size
// The entries RECORD.abbrev_offset, RECORD.desc_offset and RECORD.alias_offset
// points to the strings in this block. The values are calculated from the beginning
// of the strings-block.
//
// The code below does not use the structures above because to overcome
// problems with datatype sizes and record alignments wit the different
// compilers and operating systems. So the code may be hard to read.
// But i hope the explanation above will help a much.


static char *Priv [] = {"Disgrace",
                        "Limited",
                        "Normal",
                        "Worthy",
                        "Privil",
                        "Favored",
                        "Extra",
                        "Clerk",
                        "AsstSysop",
                        "Twit",
                        "Sysop",
                        "Hidden",
                        0};



void TMaxSeverity::init (const string & maximusPath)
{
  FILE *fsev;
  int  count;
  char *path;

  path = new char[maximusPath.length () + 12];
  strcpy (path, maximusPath.c_str ());
  strcat (path, "ACCESS.DAT");

  count = 0;
  if ((fsev = fopen (path, "rb")) != NULL)
  {                                                // Maximus detected
    unsigned char header[12];

    if ((fread (&header, 12, 1, fsev) != 0) &&
        (*header == 0x21) && (header[1] == 0x93) && (header[2] == 0x7C) && (header[3] == 0x8F))
    {
      long first_record;
      long number_records;
      long record_size;
      long data_offset;
      long data_size;
      char *buf;

      first_record = header[4] + (header[5] << 8);
      number_records = header[6] + (header[7] << 8);
      record_size = header[8] + (header[9] << 8);
      data_offset = first_record + number_records * record_size;

      data_size = header[10] + (header[11] << 8);
      buf = new char[data_size];                 // holds severity in ascii

      fseek (fsev, data_offset, SEEK_SET);
      if (fread (buf, data_size, 1, fsev) != 0)
      {
        unsigned char *record;

        record = new unsigned char[record_size];

        fseek (fsev, first_record, SEEK_SET);
        while ((count < number_records) &&
               (fread (record, record_size, 1, fsev) != 0))
        {
          unsigned short severity;
          long abbrev_offset;
          long desc_offset;
          long alias_offset;

          severity = record[0] + (record[1] << 8);
          abbrev_offset = record[4] + (record[5] << 8);
          desc_offset = record[6] + (record[7] << 8);
          alias_offset = record[8] + (record[9] << 8);

          fValues.insert (severityValueMap::value_type (severity, &buf[abbrev_offset]));
          fStrings.insert (severityStringMap::value_type (&buf[abbrev_offset], severity));
          if ((abbrev_offset != desc_offset) && (0 != desc_offset))
            fStrings.insert (severityStringMap::value_type (&buf[desc_offset], severity));

          if ((abbrev_offset != alias_offset) && (desc_offset != alias_offset) && (0 != alias_offset))
            fStrings.insert (severityStringMap::value_type (&buf[alias_offset], severity));

          count++;
        }
        delete [] record;
      }
      delete [] buf;
    }
    fclose (fsev);
  }
  delete [] path;

  if (count == 0)
  {                                  // Maximus not detected - set defaults
    for (unsigned short severity = 0; Priv[severity] != 0; severity++)
    {
      fValues.insert (severityValueMap::value_type (severity, Priv[severity]));
      fStrings.insert (severityStringMap::value_type (Priv[severity], severity));
    }
  }
}



unsigned short TMaxSeverity::ascii2severity (const string & in)
{
  severityStringMap::iterator iter;

  if ((iter = fStrings.find (in)) != fStrings.end ())
    return (*iter).second;

  return 0;
}



string TMaxSeverity::severity2ascii (unsigned short level)
{
  severityValueMap::iterator iter;

  if ((iter = fValues.find (level)) != fValues.end ())
    return (*iter).second;

  return "";
}


