#include "TTextFileWriter.h"
//#include <iomanip>
#include "TFormattedOutput.h"



TTextFileWriter::TTextFileWriter (ostream *file, bool inKBytes,
                                  char fileMarker, int wordBreak,
//                                  int addIndent,
                                  TFormattedOutput & format)
: TFileWriter (file), fInKBytes (inKBytes),
  fFileMarker (fileMarker),
  fWordBreak (wordBreak),
//  fAddIndent (addIndent),
  fFormat (&format)
{
}



void TTextFileWriter::write (const TFileEntry & entry, bool isNew)
{
//  int start;

  if (!fFile || !fFile->good ())
    return;

  if (entry.name ().length () == 0)
  {
    fFormat->setParam (TFormattedOutput::ENTRY_DESC, entry.desc ());
    fFormat->writeLine (*fFile, true);
  }
  else
  {
    fFormat->setParam (TFormattedOutput::ENTRY_NAME, entry.name ());

    if (entry.available ())
    {
      char date[9];
      sprintf (date, "%2i.%02i.%02i", entry.day (), entry.month (), entry.year () % 100);
      fFormat->setParam (TFormattedOutput::ENTRY_DATE, date);
      fFormat->setParam (TFormattedOutput::ENTRY_SIZE, fInKBytes ? (entry.size () + 1023) / 1024 : entry.size (), fInKBytes);
    }
    else
    {
      fFormat->setParam (TFormattedOutput::ENTRY_DATE, "");
      fFormat->setParam (TFormattedOutput::ENTRY_SIZE, "<unknown>");
    }
    char marker[2];
    *marker = isNew ? fFileMarker : ' ';
    marker[1] = '\0';
    fFormat->setParam (TFormattedOutput::ENTRY_NEWMARKER, marker);

    breakDescription (entry.desc ());
  }
}



void TTextFileWriter::breakDescription (const string & str)
{
  char *buffer;
  char *cont;
  char *start;

  buffer = new char[str.length () + 1];
  strcpy (buffer, str.c_str ());

  // detect continuation lines
  if ((cont = strchr (buffer, '\n')) != 0)
    *cont = '\0';

  // skip leading blancs
  for (start = buffer; (*start != '\0') && (*start == ' '); start++);

//  int indent = fStart;
  bool first = true;
  while (*start != '\0')
  {
    int offset = 0; // indent;
    char *pos = start;
    char *last;

    while ((*pos != '\0') && (offset <= fWordBreak))
    {
      last = pos;

      for (; *pos == ' '; pos++, offset++);
      for (; (*pos != '\0') && (*pos != ' '); pos++, offset++);
    }

    if (*pos == '\0')                // scan reached end of line
    {
      if (offset > fWordBreak)       // but exceeds the line limits
      {
        if (start != last)           // no word break position found ?
        {
          pos = last;
          *pos = '\0';
          pos++;
        }
      }
    }
    else                             // scan reached not end of line
    {
      pos = last;
      *pos = '\0';
      pos++;
    }

    fFormat->setParam (TFormattedOutput::ENTRY_DESC, start);
    fFormat->writeLine (*fFile);

    if (first)
    {
      fFormat->setParam (TFormattedOutput::ENTRY_NAME, "");
      fFormat->setParam (TFormattedOutput::ENTRY_SIZE, "");
      fFormat->setParam (TFormattedOutput::ENTRY_DATE, "");
      fFormat->setParam (TFormattedOutput::ENTRY_NEWMARKER, "");
      first = false;
    }

    for (start = pos; *start == ' '; start++);
  }

  if (first)
  {
    fFormat->setParam (TFormattedOutput::ENTRY_NAME, "");
    fFormat->setParam (TFormattedOutput::ENTRY_SIZE, "");
    fFormat->setParam (TFormattedOutput::ENTRY_DATE, "");
    fFormat->setParam (TFormattedOutput::ENTRY_NEWMARKER, "");
  }

  // now handle the continuation lines - copy them as they are
  while (cont)
  {
    cont++;

    char *pos;
    if ((pos = strchr (cont, '\n')) != 0)
      *pos = '\0';

    fFormat->setParam (TFormattedOutput::ENTRY_DESC, cont);
    fFormat->writeLine (*fFile);

    cont = pos;
  }
}

