#include "TWriteArea.h"
#include "TArea.h"
#include "TFileEntry.h"
#include "TFormattedOutput.h"
#include "TTextFileWriter.h"
#include "THtmlFileWriter.h"



TWriteArea::TWriteArea (ostream & fall, ostream *fnew, const string & fileMark,
                        bool inKBytes, int wordBreak, Style style,
                        time_t runTime, time_t recentNew, TFormattedOutput & format)
: fAllfilesFile (&fall), fNewfilesFile (fnew), fInKBytes (inKBytes),
  fRuntime (runTime), fRecentNew (recentNew), fFormat (&format), fNewFileWriter (0)
{
  if (style == STYLE_TEXT)
  {
    fAllFileWriter = new TTextFileWriter (&fall, inKBytes, fileMark[0], wordBreak, format);
    if (fnew && fnew->good ())
      fNewFileWriter = new TTextFileWriter (fnew, inKBytes, fileMark[0], wordBreak, format);
  }
  else
  {
    fAllFileWriter = new THtmlFileWriter (&fall, inKBytes, fileMark[0], format);
    if (fnew && fnew->good ())
      fNewFileWriter = new THtmlFileWriter (fnew, inKBytes, fileMark[0], format);
  }
}



TWriteArea::~TWriteArea ()
{
  delete fAllFileWriter;
  delete fNewFileWriter;
}



void TWriteArea::writeArea (const TArea & area, const TFileList & fileList)
{
  if (fFormat && !fFormat->empty ())
  {
    fFormat->setParam (TFormattedOutput::VERSATILE, area.fPath.c_str ());

    // first write the header of the area
    writeHeader (*fAllfilesFile, area, fileList.getCount (), fileList.getSize (), fileList.getNewestFile ());
    if (fNewfilesFile && fNewfilesFile->good () && (fileList.getNewCount () != 0))
      writeHeader (*fNewfilesFile, area, fileList.getNewCount (), fileList.getNewSize (), fileList.getNewestFile ());

    // then write the body - a list of files in this area
    TFileList::const_iterator entryIter = fileList.begin ();
    while (entryIter != fileList.end ())
    {
      bool isnew = (fRuntime - (*entryIter).fileTime ()) <= fRecentNew;

      fAllFileWriter->write (*entryIter, isnew);
      if (isnew && fNewFileWriter)
        fNewFileWriter->write (*entryIter, isnew);

      ++entryIter;
    }

    // at least write the trailer of the area
    writeFooter (*fAllfilesFile, area, fileList.getCount (), fileList.getSize (), fileList.getNewestFile ());
    if (fNewfilesFile && fNewfilesFile->good () && (fileList.getNewCount () != 0) && fNewFileWriter)
      writeFooter (*fNewfilesFile, area, fileList.getNewCount (), fileList.getNewSize (), fileList.getNewestFile ());
  }
}



void TWriteArea::writeHeader (ostream & f, const TArea & area,
                              long count, unsigned long size, const TFileEntry *latest)
{
  if (fFormat && !fFormat->empty ())
  {
    fFormat->setParam (TFormattedOutput::SECTION_NAME, area.fName);
    fFormat->setParam (TFormattedOutput::SECTION_DESC, area.fInfo);
    fFormat->setParam (TFormattedOutput::SECTION_ACCESS, area.fPriv);
    fFormat->setParam (TFormattedOutput::SECTION_COUNT, count);
    fFormat->setParam (TFormattedOutput::SECTION_SIZE, fInKBytes ? (size + 1023) / 1024 : size, fInKBytes);
    fFormat->setParam (TFormattedOutput::SECTION_LATEST, latest ? latest->name ().c_str () : "");
    fFormat->writeTop (f);
  }
  f << endl;
}



void TWriteArea::writeFooter (ostream & f, const TArea & area,
                              long count, unsigned long size, const TFileEntry *latest)
{
  if (fFormat && !fFormat->empty ())
    fFormat->writeTail (f);
}

