#include <time.h>
#include <stdio.h>
#include <string>
#include <iostream>
#include <iomanip>
#include <fstream>
#include "TAreaList.h"
#include "TAreaInfo.h"
#include "TMaxSeverity.h"
#include "TWriteArea.h"
#include "TFileList.h"
#include "TConfig.h"
#include "THtmlAreaWriter.h"



string version   = "Version 0.30";
string copywr    = "  Generated by FFL " + version + "           (c) 1993-2000 by Petric Frank";



class TFList
{
  public:
    TFList (char *confpath);
    ~TFList ();

    bool analyzeCmdLine (int argc, char *argv[]);
    int  openFiles ();
    void run ();
    void writeSummary ();

  private:
    typedef vector<THtmlAreaWriter *> areaWriterVector;

//    void generateHtmlStyle (const TArea & area, const TFileList & list);
    void createHtmlTree ();
    void createHtmlSubtree (TAreaInfoList & subtree, THtmlAreaWriter & writer, const string & section);
    void copyBoxinfo (ostream & filelist, const string & filename);
    void writeSummaryHeader (ostream & f);

    TConfig       fConfig;
    TWriteArea    *fWriteArea;
    TAreaList     *fAreas;
    TAreaInfoList *fAllareaList;
    TAreaInfoList *fNewareaList;
    time_t        fActualTime;
    string        fTimeString;
    ofstream      fAllfilesFile;
    ofstream      fNewfilesFile;
};



TFList::TFList (char *confpath)
: fConfig (confpath), fWriteArea (0), fAreas (0),
  fAllfilesFile (fConfig.getAllFiles ().c_str ()),
  fNewfilesFile (fConfig.getNewFiles ().c_str ())
{
}



TFList::~TFList ()
{
  delete fAreas;
  delete fWriteArea;
  delete fAllareaList;
  delete fNewareaList;
}



bool TFList::analyzeCmdLine (int argc, char *argv[])
{
  for (int count = 1; count < argc; count++)
    if (!(fConfig.treatCmdLine (argv[count])))
    {
      cout << "Parameter in error : " << argv[count] << endl;
      return false;
    }

  return true;
}



int TFList::openFiles ()
{
  string path;
  time_t runtime;

  path = fConfig.getMaxDir ();
  fAreas = new TAreaList (path, fConfig.isSorted (), fConfig.getAreas (),
                          fConfig.getAddAreas (), fConfig.defaultSorting ());
  if (fAreas->size () == 0)
  {
    cout << "Error : No file area specification found." << endl;
    cout << "        None of the following reasons are fulfiled:" << endl;
    cout << "        - " << path << "FAREA.DAT was found with defined file areas" << endl;
    cout << "        - " << path << "AREA.DAT was found with defined file areas" << endl;
    cout << "        - " << "the configuration file don't define file areas" << endl;
    return 2;
  }

  if (!fAllfilesFile.good ())
  {
    cout << "Error : file " << fConfig.getAllFiles ()
         << " could not be opended" << endl;
    return 3;
  }

  if (fConfig.getNewFiles ().length () != 0)
  {
    if (!fNewfilesFile.good ())
    {
      cout << "Error : file " << fConfig.getNewFiles ()
           << " could not be opended" << endl;
      return 4;
    }
  }

  time (&fActualTime);                           // preserve run date & time
  char buffer[20];
  strftime (buffer, 79, "%d.%m.%Y %R", localtime (&fActualTime));
  fTimeString = buffer;

  fWriteArea = new TWriteArea (fAllfilesFile, &fNewfilesFile, fConfig.newFileMarker (),
                               fConfig.areInKBytes (), fConfig.getWordBreak (),
                               TWriteArea::STYLE_TEXT, fActualTime,
                               fConfig.getRecent (), fConfig.getTextFileAreaFormat ());

  copyBoxinfo (fAllfilesFile, fConfig.getAllHeader ());
  fAllfilesFile << endl << endl << "Generated at : " << fTimeString << endl << endl << endl;
  fAllareaList = new TAreaInfoList (fConfig.getSummaryDef ());
  if (fNewfilesFile.good ())
  {
    copyBoxinfo (fNewfilesFile, fConfig.getNewHeader ());
    fNewfilesFile << endl << endl << "Generated at : " << fTimeString << endl << endl << endl;
    fNewareaList = new TAreaInfoList (fConfig.getSummaryDef ());
  }

  // set some predefined values to output format definition lists
  fConfig.getTextFileAreaFormat ().setParam (TFormattedOutput::CREATION_DATE, buffer);
  fConfig.getTextFileAreaFormat ().setParam (TFormattedOutput::BBS_NAME, fConfig.getBbsName ().c_str ());
  fConfig.getTextFileAreaFormat ().setParam (TFormattedOutput::COPYRIGHT, copywr.c_str ());
  fConfig.getHtmlFileAreaFormat ().setParam (TFormattedOutput::CREATION_DATE, buffer);
  fConfig.getHtmlFileAreaFormat ().setParam (TFormattedOutput::BBS_NAME, fConfig.getBbsName ().c_str ());
  fConfig.getHtmlFileAreaFormat ().setParam (TFormattedOutput::COPYRIGHT, copywr.c_str ());
  fConfig.getHtmlAreaFormat ().setParam (TFormattedOutput::CREATION_DATE, buffer);
  fConfig.getHtmlAreaFormat ().setParam (TFormattedOutput::BBS_NAME, fConfig.getBbsName ().c_str ());
  fConfig.getHtmlAreaFormat ().setParam (TFormattedOutput::COPYRIGHT, copywr.c_str ());
  fConfig.getSummaryDef ().setParam (TFormattedOutput::CREATION_DATE, buffer);
  fConfig.getSummaryDef ().setParam (TFormattedOutput::BBS_NAME, fConfig.getBbsName ().c_str ());
  fConfig.getSummaryDef ().setParam (TFormattedOutput::COPYRIGHT, copywr.c_str ());

  return 0;
}



void TFList::run ()
{
  TArea         areaData;

  fAreas->locateFirstArea ();
  while (fAreas->getArea (areaData, fConfig.getAddAreas ()))
  {
    if (TMaxSeverity::ascii2severity (areaData.fPriv) <= fConfig.getPriv ())
    {
      cout << "\rArea " << setiosflags (ios::left) << setw (16) << areaData.fName << " : " << areaData.fInfo << endl;

      // first retrieve the files belonging to this area
      TFileList fileList (areaData.fPath, areaData.fList, fConfig.getSkipLines (),
                          fConfig.mustPurgeSize (), fActualTime,
                          fConfig.getRecent (), areaData.fSortInfo,
                          fConfig.getMultilineTag (), fConfig.getMultilineIndent ());

      if (fileList.size () != 0)
      {
        // append them to the all-files and new-files lists
        fWriteArea->writeArea (areaData, fileList);

        // generate the area in HTML style - if requested
        if (fConfig.getGenerateHtml ())
        {
          string fileName;

          fileName = fConfig.getHtmlFilesDir () + areaData.fName + ".html";
          ofstream htmlFile (fileName.c_str ());
          if (htmlFile.good ())
          {
            TArea newArea = areaData;
            fConfig.getHtmlDirMapper ().mapPath (areaData.fPath, newArea.fPath);

            TWriteArea htmlArea = TWriteArea (htmlFile, 0, fConfig.newFileMarker (),
                                              fConfig.areInKBytes (), fConfig.getWordBreak (),
                                              TWriteArea::STYLE_HTML, fActualTime,
                                              fConfig.getRecent (), fConfig.getHtmlFileAreaFormat ());

            htmlArea.writeArea (newArea, fileList);
          }
        }

        // update statistics info (needed for summary info)
        fAllareaList->add (TAreaInfo (areaData, fileList.getCount (), fileList.getSize ()));
        if ((fWriteArea->newfilesFile ()) && (fileList.getNewCount () > 0))
          fNewareaList->add (TAreaInfo (areaData, fileList.getNewCount (), fileList.getNewSize ()));
      }
    }
  }
  cout << "\r               " << endl;

  writeSummary ();

  createHtmlTree ();

  cout << "   Number of Files : " << fAllareaList->getCount () << endl;
  cout << "   Used Space      : " << fAllareaList->getSize () << endl;
}



void TFList::createHtmlTree ()
{
  if (fConfig.getGenerateHtml ())
  {
    cout << "                 " << endl
         << "Writing HTML Tree" << endl << endl;

    THtmlAreaWriter allAreas (fConfig.getHtmlFilesDir () + "allfiles.html",
                              "All Files", "All Files of " + fConfig.getBbsName (),
                              fConfig.getHtmlAreaFormat ());

    createHtmlSubtree (*fAllareaList, allAreas, "");
  }
}



void TFList::createHtmlSubtree (TAreaInfoList & subTree, THtmlAreaWriter & writer, const string & section)
{
  int loop;

  bool *isHandled = new bool[subTree.size ()];

  for (loop = 0; loop < subTree.size (); loop++)
    isHandled[loop] = false;

  if (writer.open (subTree.getCount (), subTree.getSize ()));
  {
    for (loop = 0; loop < subTree.size (); loop++)
    {
      if (!isHandled[loop])
      {
        string area = subTree[loop].name ();
        string desc;

        int pos = area.find_first_of ('.', section.length () + 1);
        if (pos == string::npos)
        {
          unsigned long size = (subTree[loop].size () + 1023) / 1024;

          desc = fAreas->divisionDescription (area);

          writer.writeEntry (area, area + ".html", desc, subTree[loop].count (), size);

          isHandled[loop] = true;
        }
        else
        {
          TAreaInfoList subTree2 (fConfig.getSummaryDef ());

          string section2 = area.substr (0, pos);
          desc = fAreas->divisionDescription (section2);

          string comp = area.substr (0, pos + 1);
          for (int copy = loop; copy < subTree.size (); copy++)
          {
            if (comp == subTree[copy].name ().substr (0, comp.length ()))
            {
              subTree2.add (subTree[copy]);

              isHandled[copy] = true;
            }
          }

          THtmlAreaWriter writer2 (fConfig.getHtmlFilesDir () + section2 + ".html",
                                   section2, desc, fConfig.getHtmlAreaFormat ());

          createHtmlSubtree (subTree2, writer2, section2);

          writer.writeEntry (section2, section2 + ".html", desc, subTree2.getCount (), subTree2.getSize ());
        }
      }
    }
  }

  delete [] isHandled;
}



void TFList::writeSummary ()
{
  if (!fConfig.hasNoSummary ())
  {
    fAllareaList->write (fAllfilesFile, fConfig.areInKBytes ());
    if (fNewfilesFile.good ())
      fNewareaList->write (fNewfilesFile, fConfig.areInKBytes ());
  }

  fAllfilesFile << endl << copywr << endl;
  if (fNewfilesFile.good ())
    fNewfilesFile << endl << copywr << endl;
}



void TFList::copyBoxinfo (ostream & filelist, const string & filename)
{
  ifstream fBoxInfo (filename.c_str (), ios::in);
  if (fBoxInfo)
  {
    string line;

    while (fBoxInfo.good ())
    {
      getline (fBoxInfo, line);
      filelist << line << endl;
    }
  }
}



void printExplanation ()
{
  cout
  << "Command line : "
  << "FFL"
  << " <parameters>" << endl << endl
  << "Valid Parameters :" << endl
  << "  /B<size>    Wordbreak in comments at <size> (min. 20)." << endl
  << "  /C<cfile>   Filename of configuration file." << endl
  << "  /D<days>    Recent <days> (default 7) files will be shown as new." << endl
  << "  /KB         Displays the file sizes in KBytes." << endl
  << "  /IA<path>   Path to ALLFILES information file to be placed at the top." << endl
  << "  /LA<path>   Path to the ALL-listfile to be generated." << endl
  << "  /IN<path>   Path to NEWFILES information file to be placed at the top." << endl
  << "  /LN<path>   Path to the NEW-listfile to be generated." << endl
  << "  /M<dir>     Directory where AREA.DAT of Maximus is located." << endl
  << "  /NA         do not sort file areas by areaname." << endl
  << "  /NS         If you don't want to get the summary at the end of the list" << endl
  << "  /PS         supress Filesizes (at first position in desc area in FILES.BBS)." << endl
  << "  /S<priv>    Top security level to be listed." << endl << endl;
}



int main (int argc, char *argv[])
{
  TFList *flist;
  int    rc;
  int    found;
  int    count;

  cout << endl
       << "Maximus filelist generator  "
       << version
       << "         (c) 1993-2000 by Petric Frank"
       << endl << endl;

  if ((argc == 2) && (strncmp (argv[1], "/?", 2) == 0))
  {
    printExplanation ();
    return 255;
  }

  for (count = 1, found = 0; (count < argc) && (found == 0); count++)
    if (strnicmp (argv[count], "/C", 2) == 0)
      found = 1;

  if (found == 1)
    flist = new TFList (&((argv[count - 1])[2]));
  else
    flist = new TFList ("");

  cout << endl;

  if (!(flist->analyzeCmdLine (argc, argv)))
    return 1;

  if ((rc = flist->openFiles ()) != 0)
    return rc;

  flist->run ();

  cout << endl << "Filelist written" << endl;

  return 0;
}


